@@ -29,9 +29,8 @@ daviesLM<-function(y, z, xreg, weights, offs, values, k, alternative){
        y<-y-offs
        n<-length(y)
        n1<-length(values)
-       RIS<-matrix(,n1,2)
-
-       X.psi<-matrix(,n,length(fn))
+       RIS<-matrix(NA,n1,2)
+       X.psi<-matrix(NA,n,length(fn))
        df.res<- n - ncol(xreg) - length(fn)
        for(i in 1:n1){
           for(j in 1:length(fn)) X.psi[,j]<-eval(parse(text=fn[[j]]), list(x=z, p=values[i]))
@@ -43,7 +42,7 @@ daviesLM<-function(y, z, xreg, weights, offs, values, k, alternative){
           #RIS[i]<-((rss0-rss1)/ncol(X.psi))/sigma2
           #Wald
           invXtX1<-try(solve(crossprod(sqrt(weights)*xx1.new)), silent=TRUE)
-          if(class(invXtX1)!="try-error"){
+          if(class(invXtX1)[1]!="try-error"){
             hat.b<-drop(invXtX1%*%crossprod(weights*xx1.new,y))
             mu1.new<-xx1.new%*%hat.b
             devE<-sum((weights*(y-mu1.new)^2))
@@ -206,7 +205,11 @@ daviesGLM<-function(y, z, xreg, weights, offs, values=NULL, k, list.glm, alterna
     mf$formula<-update.formula(mf$formula,paste(seg.Z,collapse=".+"))
     formulaOrig<-formula(obj)
     if(class(obj)[1]=="segmented"){
-        if(!is.null(eval(obj$call$obj)$call$data)) mf$data <- eval(obj$call$obj)$call$data
+        #10/12/20. se segmented e' stato chiamato con un oggetto m (cioe m<-lm()), allora digitando eval(obj$call$obj)
+        #restituiva 'm' ottenuto dal comando di sopra.. match(c("formula",..
+        #sembra che mettendo eval(obj$call$obj, parent.frame()) o anche eval(obj$call$obj, envir=0) riesca ad ignorare 
+        #l'oggetto m che si trova nell'ambiente della funzione  
+        if(!is.null(eval(obj$call$obj, parent.frame())$call$data)) mf$data <- eval(obj$call$obj, parent.frame())$call$data
         mf$formula<-update.formula(mf$formula,paste("~.-",paste(obj$nameUV$V, collapse="-")))
         for(i in 1:length(obj$nameUV$U)) assign(obj$nameUV$U[i], obj$model[,obj$nameUV$U[i]], envir=parent.frame())
         formulaOrig<-update.formula(formulaOrig, paste("~.-",paste(obj$nameUV$V, collapse="-")))
