@@ -30,12 +30,32 @@ seg.Ar.fit<-function(obj, XREG, Z, PSI, opz, return.all.sol=FALSE){
         isOK<- all(is.ok) && all(!is.na(is.ok))
         isOK}
     #------------
-    far.psi<-function(Z, PSI, id.psi.group, ret.id=TRUE) {
+    far.psi<-function(Z, PSI, id.psi.group, ret.id=TRUE, fc=.93) {
+        #check if psi are far from the boundaries ..s
+        #   returns TRUE, if fine.
         #id.far.ok<-sapply(unique(id.psi.group), function(.x) (table(rowSums(((Z>PSI)[,id.psi.group==.x,drop=FALSE])))>=2)[-1]) #[-1] esclude lo zero, x<psi[1] 
-        id.far.ok<-sapply(unique(id.psi.group), function(.x) (tabulate(rowSums(((Z>PSI)[,id.psi.group==.x,drop=FALSE]))+1)>=2)[-1]) #[-1] esclude lo zero, x<psi[1]
+        #id.far.ok<-sapply(unique(id.psi.group), function(.x) (tabulate(rowSums(((Z>PSI)[,id.psi.group==.x,drop=FALSE]))+1)>=2)[-1]) #[-1] esclude lo zero, x<psi[1]
+        #16/01/20:
+        # se un psi assume l'estremo superiore "Z>PSI" non se ne accorge, mentre Z>=PSI, si.. Il contrario e vero con estremo inf e Z>PSI
+        nSeg<-length(unique(id.psi.group))
+        npsij<-tapply(id.psi.group,id.psi.group,length)
+        nj<-sapply(unique(id.psi.group), function(.x) { tabulate(rowSums((Z>PSI)[,id.psi.group==.x,drop=FALSE])+1) }, simplify = FALSE)    
+        ff<-id.far.ok<-vector("list",length=nSeg) 
+        for(i in 1:nSeg){
+            if(length(nj[[i]])!=npsij[i]+1) nj[[i]]<- tabulate(rowSums((Z>=PSI)[,id.psi.group==i,drop=FALSE])+1)
+            id.ok<-(nj[[i]] >= 2)
+            id.far.ok[[i]] <- id.ok[-length(id.ok)] & id.ok[-1] #esattamente uguale al numero di psi del gruppo i
+            ff[[i]]<-ifelse(diff(nj[[i]])>0, 1/fc, fc)
+        }
         id.far.ok<-unlist(id.far.ok)
-        if(ret.id) return(id.far.ok) else return(all(id.far.ok))
-    }
+        ff<-unlist(ff)
+        if(!ret.id) {return(all(id.far.ok))
+        } else {
+            attr(id.far.ok,"factor") <- ff
+            return(id.far.ok) 
+        }
+        #if(ret.id) return(id.far.ok) else return(all(id.far.ok))
+    } #end far.psi
     #-----------
     adj.psi<-function(psii, LIM) {pmin(pmax(LIM[1,],psii),LIM[2,])} 
     #-----------
@@ -61,6 +81,7 @@ seg.Ar.fit<-function(obj, XREG, Z, PSI, opz, return.all.sol=FALSE){
     id.psi.group<-opz$id.psi.group
     it.max<-old.it.max<-opz$it.max
     psi<-PSI[1,]
+    fc<-opz$fc
     names(psi)<-id.psi.group
     epsilon <- 10
     dev.values<-psi.values <- NULL
@@ -112,6 +133,7 @@ seg.Ar.fit<-function(obj, XREG, Z, PSI, opz, return.all.sol=FALSE){
     }
     #==============================================  
     id.warn <- FALSE
+    id.psi.changed<-rep(FALSE, it.max)
     while (abs(epsilon) > toll) {
         it<-it+1
         n.psi0 <- n.psi1
@@ -208,23 +230,20 @@ seg.Ar.fit<-function(obj, XREG, Z, PSI, opz, return.all.sol=FALSE){
         psi.values[[length(psi.values) + 1]] <- psi
         dev.values[length(dev.values) + 1] <- L0
         
-        if (it >= it.max) {
-            id.warn <- TRUE
-            break
-        }
         #Mi sa che non servono i controlli.. soprattutto se non ha fatto step-halving
         #check if i psi ottenuti sono nel range o abbastanza lontani
-        id.psi.far <-far.psi(Z, PSI, id.psi.group, TRUE)
+        id.psi.far <-far.psi(Z, PSI, id.psi.group, TRUE, fc=opz$fc)
         id.psi.in <- in.psi(limZ, PSI, TRUE)
         id.psi.ok <- id.psi.in & id.psi.far 
         
         if(!all(id.psi.ok)){
             if(fix.npsi){
-                psi<-adj.psi(psi, limZ) #within range!!!
-                id.psi.far<-far.psi(Z, PSI, id.psi.group, TRUE)
+#                psi<-adj.psi(psi, limZ) #within range!!!
+#                id.psi.far<-far.psi(Z, PSI, id.psi.group, TRUE)
                 psi<-psi*ifelse(id.psi.far,1,.9) 
                 PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
-            } else {
+                id.psi.changed[it]<-TRUE
+                    } else {
                 Z<-Z[, id.psi.ok, drop=FALSE]
                 PSI<-PSI[, id.psi.ok, drop=FALSE]
                 rangeZ <- rangeZ[,id.psi.ok,drop=FALSE]
@@ -240,8 +259,14 @@ seg.Ar.fit<-function(obj, XREG, Z, PSI, opz, return.all.sol=FALSE){
                 }
             }
         }
+        if (it >= it.max) {
+            id.warn <- TRUE
+            break
+        }
     } #end while_it
     ##=============================================================================
+    if(id.psi.changed[length(id.psi.changed)]) warning(paste("Some psi (", (1:length(psi))[!id.psi.far],
+                                                             ") changed after the last iter.",sep=""), call. = FALSE)
     if(id.warn) warning(paste("max number of iterations (", it,") attained",sep=""), call. = FALSE)
     
     attr( psi.values, "dev") <- dev.values
@@ -257,7 +282,7 @@ seg.Ar.fit<-function(obj, XREG, Z, PSI, opz, return.all.sol=FALSE){
     PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
     
     #U e V possono essere cambiati (rimozione/ordinamento psi.. ) per cui si deve ricalcolare il tutto, altrimenti sarebbe uguale a U1 e obj1
-    if(sd(PSI-PSI.old)>0){
+    if(sd(PSI-PSI.old)>0 || id.psi.changed[length(id.psi.changed)]){
         U <- (Z-PSI)*(Z>PSI)
         colnames(U)<-paste("U", 1:ncol(U), sep = "")
         V <- -(Z>PSI)
