@@ -1,4 +1,4 @@
-segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), model = TRUE, 
+segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model = TRUE, 
     keep.class = FALSE, ...) {
 
 #if("|" %in% all.names(formula(obj))) {
@@ -9,6 +9,24 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
 #    Fo <- as.formula(Fo.charac)
 #    Fo.conDisp <- as.formula(paste(Fo.charac,nomiX.disp,sep="|"))
     #}
+  build.all.psi<-function(psi, fixed.psi){
+    all.names.psi<-union(names(psi),names(fixed.psi))
+    all.psi<-vector("list", length=length(all.names.psi))
+    names(all.psi)<- all.names.psi
+    for(i in names(all.psi)) {
+      if(!is.null(psi[[i]])){
+        psi[[i]]<-sort(psi[[i]])
+        names(psi[[i]])<-paste("U",1:length(psi[[i]]),".",i,sep="")
+      }
+      if(!is.null(fixed.psi[[i]])){
+        fixed.psi[[i]]<-sort(fixed.psi[[i]])
+        names(fixed.psi[[i]])<-	paste("U",1:length(fixed.psi[[i]]),".fixed.",i,sep="")
+      }
+      all.psi[[i]]<-sort(c(psi[[i]],fixed.psi[[i]]))
+    }
+    return(all.psi)
+  }
+  ##===inizio funzione
 
     update.formula1<-function(old,new,...,opt=1){
     #se old e' una formula che contiene "|", questa funzione aggiorna old con new, 
@@ -45,6 +63,11 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
             seg.Z <- as.formula(paste("~", all.vars(formula(obj))[2]))
         else stop("please specify 'seg.Z'")
     }
+  
+    if("V" %in% sub("V[1-9]*[0-9]","V", c(all.vars(seg.Z), all.vars(formula(obj) )[-1]))) stop("variable names 'V', 'V1', .. are not allowed")
+    if("U" %in% sub("U[1-9]*[0-9]","U", c(all.vars(seg.Z), all.vars(formula(obj) )[-1]))) stop("variable names 'U', 'U1', .. are not allowed")
+    if(any(c("$","[") %in% all.names(seg.Z))) stop(" '$' or '[' not allowed in 'seg.Z' ")
+    
     n.Seg <- length(all.vars(seg.Z))
     id.npsi <- FALSE
     if (missing(psi)) {
@@ -55,20 +78,19 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
             if (length(npsi) != length(all.vars(seg.Z))) 
                 stop("seg.Z and npsi do not match")
             names(npsi) <- all.vars(seg.Z)
-        }
-        else {
-            if (missing(npsi)) 
-                stop(" with multiple segmented variables in seg.Z, 'psi' or 'npsi' should be supplied", 
-                  call. = FALSE)
-            if (length(npsi) != n.Seg) 
-                stop(" 'npsi' and seg.Z should have the same length")
-            if (!all(names(npsi) %in% all.vars(seg.Z))) 
-                stop(" names in 'npsi' and 'seg.Z' do not match")
+        } else {
+            #if (missing(npsi)) 
+            #    stop(" with multiple segmented variables in seg.Z, 'psi' or 'npsi' should be supplied", call. = FALSE)
+            if (missing(npsi)) {
+                npsi<-rep(1, n.Seg)
+                names(npsi)<-all.vars(seg.Z)
+                }
+            if (length(npsi) != n.Seg) stop(" 'npsi' and seg.Z should have the same length")
+            if (!all(names(npsi) %in% all.vars(seg.Z))) stop(" names in 'npsi' and 'seg.Z' do not match")
         }
         psi <- lapply(npsi, function(.x) rep(NA, .x))
         id.npsi <- TRUE
-    }
-    else {
+    } else {
         if (n.Seg == 1) {
             if (!is.list(psi)) {
                 psi <- list(psi)
@@ -84,14 +106,13 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
                 stop("Names in `seg.Z' and `psi' do not match")
         }
     }
+    fc<- min(max(abs(control$fc),.8),1)       
     min.step <- control$min.step
     alpha <- control$alpha
     it.max <- old.it.max <- control$it.max
     digits <- control$digits
     toll <- control$toll
-    if (toll < 0) 
-        stop("Negative tolerance ('tol' in seg.control()) is meaningless", 
-            call. = FALSE)
+    if (toll < 0)  stop("Negative tolerance ('tol' in seg.control()) is meaningless", call. = FALSE)
     visual <- control$visual
     stop.if.error <- fix.npsi <- control$fix.npsi
     n.boot <- control$n.boot
@@ -115,8 +136,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
             visual <- FALSE
             visualBoot <- TRUE
         }
-        if (!stop.if.error) 
-            stop("Bootstrap restart only with a fixed number of breakpoints")
+        if (!stop.if.error) stop("Bootstrap restart only with a fixed number of breakpoints")
     }
     last <- control$last
     K <- control$K
@@ -132,10 +152,9 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
         mf$formula <- eval(mf$formula)
     #mf$formula <- update.formula(mf$formula, paste(seg.Z, collapse = ".+"))
     mf$formula <- update.formula1(mf$formula, paste(seg.Z, collapse = ".+"), opt=2)
+
     mfExt <- mf
     
-    
-    
     if (!is.null(obj$call$offset) || !is.null(obj$call$weights) || 
         !is.null(obj$call$subset) || !is.null(obj$call$id)) {
         mfExt$formula <- update.formula(mf$formula, paste(".~.+", 
@@ -143,6 +162,9 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
                 all.vars(obj$call$subset), all.vars(obj$call$id)), 
                 collapse = "+")))
     }
+    
+    if(!is.null(obj$call$random)) mfExt$formula<-update.formula(mf$formula, paste(".~.+", paste(all.vars(obj$call$random), collapse="+")))
+    
     mf <- eval(mf, parent.frame())
     n <- nrow(mf)
     nomiOff <- setdiff(all.vars(formula(obj)), names(mf))
@@ -153,7 +175,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
     nomiNO <- NULL
     for (i in nomiTUTTI) {
         r <- try(eval(parse(text = i), parent.frame()), silent = TRUE)
-        if (class(r) != "try-error" && length(r) == 1 && !is.function(r)) 
+        if (class(r) != "try-error" && length(r) == 1 && !is.function(r) && !i%in%names(mf)) 
             nomiNO[[length(nomiNO) + 1]] <- i
         }
     if (!is.null(nomiNO)) 
@@ -173,18 +195,19 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
     name.Z <- names(mfExt)[id.seg]
     Z <- mfExt[, id.seg, drop = FALSE]
     n.psi <- length(unlist(psi))
-    if (ncol(Z) == 1 && is.vector(psi) && (is.numeric(psi) || 
-        is.na(psi))) {
+
+    if (ncol(Z) == 1 && is.vector(psi) && (is.numeric(psi) || is.na(psi))) {
         psi <- list(as.numeric(psi))
         names(psi) <- name.Z
     }
-    if (!is.list(psi) || is.null(names(psi))) 
-        stop("psi should be a *named* list")
-    id.nomiZpsi <- match(colnames(Z), names(psi))
-    if ((ncol(Z) != length(psi)) || any(is.na(id.nomiZpsi))) 
-        stop("Length or names of Z and psi do not match")
-    nome <- names(psi)[id.nomiZpsi]
-    psi <- psi[nome]
+    id.psi <- match(colnames(Z), names(psi))
+    if ((ncol(Z) != length(psi)) || any(is.na(id.seg))) stop("Length or names of Z and psi do not match")
+    
+    
+    #nome <- names(psi)[id.psi]
+    #psi <- psi[nome]
+    
+    
     if (id.npsi) {
         for (i in 1:length(psi)) {
             K <- length(psi[[i]])
@@ -212,12 +235,37 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
                 }
         }
     }
+  #########==================== SE PSI FIXED
+  id.psi.fixed <- FALSE
+  if(!is.null(fixed.psi)){
+    id.psi.fixed <- TRUE
+    if(is.numeric(fixed.psi) && n.Seg==1) {
+      fixed.psi<-list(fixed.psi)
+      names(fixed.psi)<-all.vars(seg.Z)
+    }
+    if(is.list(fixed.psi)) {
+      if(!(names(fixed.psi) %in% all.vars(seg.Z))) stop("names(fixed.psi) is not a subset of variables in 'seg.Z' ")
+    } else {
+      stop(" 'fixed.psi' has to be a named list ")
+      } 
+    fixed.psi<-lapply(fixed.psi, sort)
+    Zfixed<-matrix(unlist(mapply(function(x,y)rep(x,y),Z[names(fixed.psi)], sapply(fixed.psi, length), SIMPLIFY = TRUE)), nrow=n)
+    n.fixed.psi<-sapply(fixed.psi, length)
+    rip.nomi <- rep( names(fixed.psi), n.fixed.psi)
+    rip.numeri <- unlist(lapply(n.fixed.psi, function(.x) 1:.x))
+    colnames(Zfixed) <- paste("U", rip.numeri,".fixed.",rip.nomi, sep="")
+    PSI <- matrix(unlist(fixed.psi), ncol=ncol(Zfixed), nrow=n, byrow = TRUE)
+    fixedU<-(Zfixed-PSI)*(Zfixed>PSI)
+    #XREG<-cbind(XREG, fixedU)
+  }
+  #########====================END  SE PSI FIXED
+
     initial.psi <- psi
     a <- sapply(psi, length)
     id.psi.group <- rep(1:length(a), times = a)
     Z <- matrix(unlist(mapply(function(x, y) rep(x, y), Z, a, 
         SIMPLIFY = TRUE)), nrow = n)
-    colnames(Z) <- nomiZ.vett <- rep(nome, times = a)
+    colnames(Z) <- nomiZ.vett <- rep(name.Z, times = a)
     psi <- unlist(psi)
     psi <- unlist(tapply(psi, id.psi.group, sort))
     k <- ncol(Z)
@@ -226,8 +274,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
     c2 <- apply((Z >= PSI), 2, all)
     if (sum(c1 + c2) != 0 || is.na(sum(c1 + c2))) 
         stop("starting psi out of the admissible range")
-    ripetizioni <- as.vector(unlist(tapply(id.psi.group, id.psi.group, 
-        function(x) 1:length(x))))
+    ripetizioni <- as.vector(unlist(tapply(id.psi.group, id.psi.group, function(x) 1:length(x))))
     nomiU <- paste("U", ripetizioni, sep = "")
     nomiU <- paste(nomiU, nomiZ.vett, sep = ".")
     nomiV <- paste("V", ripetizioni, sep = "")
@@ -242,10 +289,16 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
         mfExt[nomiU[i]] <- U[, i]
         mfExt[nomiV[i]] <- V[, i]
     }
-    Fo <- update.formula1(formula(obj), as.formula(paste(".~.+", 
-        paste(nnomi, collapse = "+"))), opt=1)
-    Fo.noV <- update.formula1(formula(obj), as.formula(paste(".~.+", 
-        paste(nomiU, collapse = "+"))), opt=1)
+
+    if(id.psi.fixed){
+        for(i in 1:ncol(fixedU)) mfExt[colnames(fixedU)[i]]<-fixedU[,i] #mf[colnames(fixedU)[i]]<-  mf e' definito sopra... Ma serve???
+        Fo <- update.formula1(formula(obj), as.formula(paste(".~.+", paste(c(nnomi,colnames(fixedU)), collapse = "+"))), opt=1)
+        Fo.noV <- update.formula1(formula(obj), as.formula(paste(".~.+", paste(c(nomiU,colnames(fixedU)), collapse = "+"))), opt=1)
+    } else {
+        Fo <- update.formula1(formula(obj), as.formula(paste(".~.+", paste(nnomi, collapse = "+"))), opt=1)
+        Fo.noV <- update.formula1(formula(obj), as.formula(paste(".~.+", paste(nomiU, collapse = "+"))), opt=1)
+    }
+    
     call.ok <- update(obj, Fo, evaluate = FALSE, data = mfExt)
     call.noV <- update(obj, Fo.noV, evaluate = FALSE, data = mfExt)
     if (it.max == 0) {
@@ -276,6 +329,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
     opz$nomiU <- nomiU
     opz$nomiV <- nomiV
     opz$fn.obj <- fn.obj
+    opz$fc=fc
     opz <- c(opz, ...)
     if (n.boot <= 0) {
         obj <- seg.def.fit(obj, Z, PSI, mfExt, opz)
@@ -284,6 +338,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
         obj <- seg.def.fit.boot(obj, Z, PSI, mfExt, opz, n.boot = n.boot, 
             size.boot = size.boot, random = random)
     }
+    
     if (!is.list(obj)) {
         warning("No breakpoint estimated", call. = FALSE)
         return(obj0)
@@ -303,6 +358,16 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
             " "), call. = FALSE)
     it <- obj$it
     psi <- obj$psi
+    #########SE PSI FIXED
+
+    psi.list<-vector("list", length=length(unique(name.Z)))
+    names(psi.list)<-name.Z
+    names(psi)<-nomiZ.vett
+    for(i in names(psi.list)){
+      psi.list[[i]]<-psi[names(psi)==i]
+    }
+
+
     psi.values <- if (n.boot <= 0) 
         obj$psi.values
     else obj$boot.restart
@@ -325,13 +390,11 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
     obj <- obj$obj
     k <- length(psi)
     
-    #browser()
-    
     #coef(obj) ha gia i nomi corretti... 
     #all.coef <- coef(obj)
     #names(all.coef) <- c(names(coef(obj0)), nomiU, nomiVxb)
     #beta.c <- all.coef[nomiU]
-    beta.c<-coef(obj)[nomiU]
+    beta.c<-unlist( unique(coef(obj)[nomiU])) #beta.c<-coef(obj)[nomiU]
     Vxb <- V %*% diag(beta.c, ncol = length(beta.c))
     nnomi <- c(nomiU, nomiVxb)
     for (i in 1:ncol(U)) {
@@ -339,11 +402,24 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
         mfExt[nomiVxb[i]] <- mf[nomiVxb[i]] <- Vxb[, i]
     }
     Fo <- update.formula1(formula(obj0), as.formula(paste(".~.+", paste(nnomi, collapse = "+"))), opt=1)
+    ###############
+
+
+
+
+
+    ############# DA GUARDARE!!!!!!!!!!!!!!!!!!!!
+    #se ci sono fixed.psi
+    if(id.psi.fixed){
+      for(i in 1:ncol(fixedU)) mfExt[colnames(fixedU)[i]]<-mf[colnames(fixedU)[i]]<-fixedU[,i]
+      Fo<-update.formula(Fo, paste(c("~.",colnames(fixedU)), collapse="+"))
+    }
+
+
+    #############
     objF <- update(obj0, Fo, evaluate = FALSE, data = mfExt)
-    if (!is.null(objF[["subset"]])) 
-        objF[["subset"]] <- NULL
-    if (is.null(opz$constr)) 
-        opz$constr <- 0
+    if (!is.null(objF[["subset"]]))  objF[["subset"]] <- NULL
+    if (is.null(opz$constr)) opz$constr <- 0
     if ((opz$constr %in% 1:2) && class(obj0) == "rq") {
         objF$method <- "fnc"
         objF$R <- quote(R)
@@ -376,23 +452,27 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
             return(objF)
         }
     }
-    
     #4/12/19: modifica fatta per consentire betareg.. Attenzione
     #semplicemente controlla se la componente "coef*" e' una lista o no..
     #COSA succede con geese models?
-    if(!is.list(objF[[grep("coef", names(objF), value = TRUE)]])){
-            names.coef <- names(coef(objF))
-            names(obj[[grep("coef", names(obj), value = TRUE)]]) <- names(objF[[grep("coef", names(objF), value = TRUE)]])
-            objF[[grep("coef", names(objF), value = TRUE)]][names.coef] <- coef(obj)[names.coef]
-        } else {
+    #
+    #giugno 20: aggiunto un tentativo "estimate" per consentire oggetti censReg 
+    nomeCoef<-grep("coef", names(objF), value = TRUE)
+    if(length(nomeCoef)==0) nomeCoef<-grep("estimate", names(objF), value = TRUE)
+    if(!is.list(objF[[nomeCoef]])){
+            objF[[nomeCoef]][names(obj[[nomeCoef]])]<-obj[[nomeCoef]]
+            #names.coef <- names(coef(objF))
+            #names(obj[[nomeCoef]]) <- names(objF[[nomeCoef]])
+            #objF[[nomeCoef]][names.coef] <- coef(obj)[names.coef]
+    } else {
             #names.coef <- names(objF[[grep("coef", names(objF), value = TRUE)]][[1]])
-            names(obj[[grep("coef", names(obj), value = TRUE)]][[1]]) <- names(objF[[grep("coef", names(objF), value = TRUE)]][[1]])
-            objF[[grep("coef", names(objF), value = TRUE)]][[1]] <- 
-                obj[[grep("coef", names(obj), value = TRUE)]][[1]]
-            objF[[grep("coef", names(objF), value = TRUE)]][[2]] <- 
-                obj[[grep("coef", names(obj), value = TRUE)]][[2]]
+            names(obj[[nomeCoef]][[1]]) <- names(objF[[nomeCoef]][[1]])
+            objF[[nomeCoef]][[1]] <- obj[[nomeCoef]][[1]]
+            objF[[nomeCoef]][[2]] <- obj[[nomeCoef]][[2]]
         }
 
+    if (!is.null(objF$maximum)) 
+        objF$maximum <- obj$maximum
     if (!is.null(objF$pseudo.r.squared)) 
         objF$pseudo.r.squared <- obj$pseudo.r.squared
     if (!is.null(objF$geese$beta)) 
@@ -468,8 +548,10 @@ segmented.default<-function (obj, seg.Z, psi, npsi, control = seg.control(), mod
     objF$id.psi.group <- id.psi.group
     objF$id.warn <- id.warn
     objF$orig.call <- orig.call
-    if (model) 
-        objF$model <- mf
+    ############## PSI FIXED
+    objF$indexU<-build.all.psi(psi.list, fixed.psi)
+    ##############
+    if (model) objF$model <- mf
     if (n.boot > 0) 
         objF$seed <- employed.Random.seed
     if (keep.class) 
