@@ -1,12 +1,35 @@
 `segmented.lm` <-
-function(obj, seg.Z, psi, npsi, control = seg.control(), model = TRUE, keep.class=FALSE, ...) {
-    
+function(obj, seg.Z, psi, npsi, fixed.psi=NULL, control = seg.control(), model = TRUE, keep.class=FALSE, ...) {
+  
+  build.all.psi<-function(psi, fixed.psi){
+    all.names.psi<-union(names(psi),names(fixed.psi))
+    all.psi<-vector("list", length=length(all.names.psi))
+    names(all.psi)<- all.names.psi
+    for(i in names(all.psi)) {
+      if(!is.null(psi[[i]])){
+        psi[[i]]<-sort(psi[[i]])
+        names(psi[[i]])<-paste("U",1:length(psi[[i]]),".",i,sep="")
+      }
+      if(!is.null(fixed.psi[[i]])){
+        fixed.psi[[i]]<-sort(fixed.psi[[i]])
+        names(fixed.psi[[i]])<-	paste("U",1:length(fixed.psi[[i]]),".fixed.",i,sep="")
+      }
+      all.psi[[i]]<-sort(c(psi[[i]],fixed.psi[[i]]))
+    }
+    return(all.psi)
+  }
+  ##===inizio funzione
+  
   if(missing(seg.Z)) {
       if(length(all.vars(formula(obj)))==2) seg.Z<- as.formula(paste("~", all.vars(formula(obj))[2])) else stop("please specify 'seg.Z'")
   }
   n.Seg<-length(all.vars(seg.Z))
   id.npsi<-FALSE
-#browser()
+  
+  if("V" %in% sub("V[1-9]*[0-9]","V", c(all.vars(seg.Z), all.vars(formula(obj) )[-1]))) stop("variable names 'V', 'V1', .. are not allowed")
+  if("U" %in% sub("U[1-9]*[0-9]","U", c(all.vars(seg.Z), all.vars(formula(obj) )[-1]))) stop("variable names 'U', 'U1', .. are not allowed")
+  if(any(c("$","[") %in% all.names(seg.Z))) stop(" '$' or '[' not allowed in 'seg.Z' ")
+  
     if(missing(psi)) { 
      if(n.Seg==1){
        if(missing(npsi)) npsi<-1
@@ -14,7 +37,11 @@ function(obj, seg.Z, psi, npsi, control = seg.control(), model = TRUE, keep.clas
        if(length(npsi)!=length(all.vars(seg.Z))) stop("seg.Z and npsi do not match") 
        names(npsi)<-all.vars(seg.Z)
      } else {#se n.Seg>1
-       if(missing(npsi)) stop(" with multiple segmented variables in seg.Z, 'psi' or 'npsi' should be supplied", call.=FALSE) 
+       #if(missing(npsi)) stop(" with multiple segmented variables in seg.Z, 'psi' or 'npsi' should be supplied", call.=FALSE) 
+       if (missing(npsi)) {
+         npsi<-rep(1, n.Seg)
+         names(npsi)<-all.vars(seg.Z)
+       }
        if(length(npsi)!=n.Seg) stop(" 'npsi' and seg.Z should have the same length")
        if(!all(names(npsi) %in% all.vars(seg.Z))) stop(" names in 'npsi' and 'seg.Z' do not match")    
        }
@@ -31,7 +58,7 @@ function(obj, seg.Z, psi, npsi, control = seg.control(), model = TRUE, keep.clas
     }
 
 
-           
+    fc<- min(max(abs(control$fc),.8),1)       
     min.step<-control$min.step
     alpha<-control$alpha
     it.max <- old.it.max<- control$it.max
@@ -117,14 +144,18 @@ nomiTUTTI<-all.vars(mfExt$formula) #comprende anche altri nomi (ad es., threshol
 nomiNO<-NULL 
 for(i in nomiTUTTI){
     r<-try(eval(parse(text=i), parent.frame()), silent=TRUE)
-    if(class(r)!="try-error" && length(r)==1 && !is.function(r)) nomiNO[[length(nomiNO)+1]]<-i
+    if(class(r)!="try-error" && length(r)==1 && !is.function(r) && !i%in%names(mf)) nomiNO[[length(nomiNO)+1]]<-i
     }
 #nomiNO dovrebbe contenere i nomi delle "altre variabili" (come th in subset=x<th) 
 if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", paste( nomiNO, collapse="-"), sep=""))
 #funziona ma se c'e' qualche variabile con il nome di una funzione (ad es., "filter")
 # anche questa variabile va a finire in nomiNO e quindi poi viene eliminata
 # problema risolto con "r<-try(evalq(.."? (prima era "r<-try(eval(..")
-#----------------------------------------------------   
+#----------------------------------------------------
+    
+    #nomiAgg<- setdiff(names(mf), names(mfExt))
+    #if(length(nomiAgg)>0) mfExt$formula<-update.formula(mfExt$formula,paste(".~.+", paste( nomiAgg, collapse="+"), sep=""))
+    
     mfExt<-eval(mfExt, parent.frame())
     
     #mantieni in mfExt solo le variabili che NON ci sono in mf (cosi la funzione occupa meno spazio..)
@@ -190,10 +221,33 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
         if(any(is.na(psi[[i]]))) psi[[i]]<-if(control$quant) {quantile(Z[[i]], prob= seq(0,1,l=K+2)[-c(1,K+2)], names=FALSE)} else {(min(Z[[i]])+ diff(range(Z[[i]]))*(1:K)/(K+1))}
         }
     }
+
+  #########==================== SE PSI FIXED
+  id.psi.fixed <- FALSE
+  if(!is.null(fixed.psi)){
+    id.psi.fixed <- TRUE
+    if(is.numeric(fixed.psi) && n.Seg==1) {
+      fixed.psi<-list(fixed.psi)
+      names(fixed.psi)<-all.vars(seg.Z)
+    }
+    if(is.list(fixed.psi)) {
+      if(!(names(fixed.psi) %in% all.vars(seg.Z))) stop("names(fixed.psi) is not a subset of variables in 'seg.Z' ")
+    } else {
+      stop(" 'fixed.psi' has to be a named list ")
+      } 
+    fixed.psi<-lapply(fixed.psi, sort)
+    Zfixed<-matrix(unlist(mapply(function(x,y)rep(x,y),Z[names(fixed.psi)], sapply(fixed.psi, length), SIMPLIFY = TRUE)), nrow=n)
+    n.fixed.psi<-sapply(fixed.psi, length)
+    rip.nomi <- rep( names(fixed.psi), n.fixed.psi)
+    rip.numeri <- unlist(lapply(n.fixed.psi, function(.x) 1:.x))
+    colnames(Zfixed) <- paste("U", rip.numeri,".fixed.",rip.nomi, sep="")
+    PSI <- matrix(unlist(fixed.psi), ncol=ncol(Zfixed), nrow=n, byrow = TRUE)
+    fixedU<-(Zfixed-PSI)*(Zfixed>PSI)
+    XREG<-cbind(XREG, fixedU)
+  }
+#########====================END  SE PSI FIXED
+
     initial.psi<-psi
-    
-#    browser()    
-    
     a <- sapply(psi, length)
     #per evitare che durante il processo iterativo i psi non siano ordinati
     id.psi.group <- rep(1:length(a), times = a) #identificativo di apparteneza alla variabile
@@ -262,7 +316,7 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     Xty<-crossprod(XREG,y)
     opz<-list(toll=toll,h=h, stop.if.error=stop.if.error, dev0=dev0, visual=visual, it.max=it.max,
         nomiOK=nomiOK, id.psi.group=id.psi.group, gap=gap, visualBoot=visualBoot, pow=pow, digits=digits,invXtX=invXtX, Xty=Xty, 
-        conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step)
+        conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, fc=fc)
     if(n.boot<=0){
     obj<-seg.lm.fit(y,XREG,Z,PSI,weights,offs,opz)
     } else {
@@ -272,6 +326,7 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
         warning("No breakpoint estimated", call. = FALSE)
         return(obj0)
         }
+    
     if(obj$obj$df.residual==0) warning("no residual degrees of freedom (other warnings expected)", call.=FALSE)
     id.psi.group<-obj$id.psi.group
     nomiOK<-obj$nomiOK
@@ -304,6 +359,15 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     nomiU   <- unlist(mapply(forma.nomiU, length.psi, nomiFINALI)) #invece di un ciclo #paste("U",1:length.psi[i], ".", name.Z[i])
     nomiVxb <- unlist(mapply(forma.nomiVxb, length.psi, nomiFINALI))
 
+    #########========================= SE PSI FIXED
+    psi.list<-vector("list", length=length(unique(nomiZ)))
+    names(psi.list)<-unique(nomiZ)
+    #names(psi)<-nomiZ #se e' una procedure automatica nomiZ puo essere piu lungo dei breakpoints "rimasti" 
+    names(psi)<-rep(nomiFINALI, length.psi)
+    for(i in names(psi.list)){
+      psi.list[[i]]<-psi[names(psi)==i]
+    }
+
     #mf<-cbind(mf, mfExt) #questo creava ripetizioni..
     for(i in 1:ncol(U)) {
         mfExt[nomiU[i]]<-mf[nomiU[i]]<-U[,i]
@@ -312,28 +376,17 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     nnomi <- c(nomiU, nomiVxb)
 #    browser()
     Fo <- update.formula(formula(obj0), as.formula(paste(".~.+", paste(nnomi, collapse = "+"))))
+    #########========================= SE PSI FIXED
+    if(id.psi.fixed){
+      for(i in 1:ncol(fixedU)) mfExt[colnames(fixedU)[i]]<-mf[colnames(fixedU)[i]]<-fixedU[,i]
+      Fo<-update.formula(Fo, paste(c("~.",colnames(fixedU)), collapse="+"))
+    }
     #objF <- update(obj0, formula = Fo, data = KK)
     objF <- update(obj0, formula = Fo,  evaluate=FALSE, data = mfExt)
     #eliminiamo subset, perche' se e' del tipo subset=x>min(x) allora continuerebbe a togliere 1 osservazione 
     if(!is.null(objF[["subset"]])) objF[["subset"]]<-NULL
     objF<-eval(objF, envir=mfExt)
- 
-
-# #11/10/16 il controllo e' stato commentato in modo tale da restituire anche un oggetto lm in cui psi viene considerato fisso..    
-#    for(jj in colnames(V)) {
-#        VV<-V[, which(colnames(V)==jj), drop=FALSE]
-#        sumV<-abs(rowSums(VV))
-##        if( (any(diff(sumV)>=2) #se ci sono due breakpoints uguali
-##            || any(table(sumV)<=1)) && stop.if.error) stop("only 1 datum in an interval: breakpoint(s) at the boundary or too close each other")
-## Tolto perche' se la variabile segmented non e' ordinata non ha senso..
-##magari potresti fare un abs(diff(psi))<=.0001? ma clusterizzato..
-#        if(any(table(sumV)<=1) && stop.if.error) stop("only 1 datum in an interval: breakpoint(s) at the boundary or too close each other")
-#        }
-    #Puo' capitare che psi sia ai margini o molto vicini (e ci sono solo 1 o 2 osservazioni in qualche intervallo. Oppure ce ne 
-    #sono di piu' ma hanno gli stessi valori di x. In questo caso objF$coef puo' avere mancanti.. names(which(is.na(coef(objF))))
-
     objF$offset<- obj0$offset
-    
     isNAcoef<-any(is.na(objF$coefficients))
     if(isNAcoef){
       if(stop.if.error) {
@@ -364,10 +417,8 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     id <- match(nomiVxb, names(coef(objF)))
     vv <- if (length(id) == 1) Cov[id, id] else diag(Cov[id, id])
     #if(length(initial)!=length(psi)) initial<-rep(NA,length(psi))
-#browser()
     a<-tapply(id.psi.group, id.psi.group, length) #ho sovrascritto "a" di sopra, ma non dovrebbe servire..
     
-    #browser()
     ris.psi<-matrix(NA,length(psi),3)
     colnames(ris.psi) <- c("Initial", "Est.", "St.Err")
     rownames(ris.psi) <- nomiVxb
@@ -401,14 +452,8 @@ if(!is.null(nomiNO)) mfExt$formula<-update.formula(mfExt$formula,paste(".~.-", p
     objF$id.psi.group <- id.psi.group
     objF$id.warn <- id.warn
     objF$orig.call<-orig.call
+    objF$indexU<-build.all.psi(psi.list, fixed.psi)
     if(model)  objF$model <- mf #objF$mframe <- data.frame(as.list(KK))
-    
-#    PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
-#    SE.PSI <- matrix(rep(sqrt(vv), rep(nrow(Z), length(psi))), ncol = length(psi))
-#    X.is<-model.matrix(Fo, data=objF$model)
-#    X.is[,nomiVxb]<-pnorm((Z-PSI)/SE.PSI)%*% diag(-beta.c, ncol = length(beta.c))
-#    objF$cov.unscaled.is<-crossprod(X.is)
-#browser()    
     if(n.boot>0) objF$seed<-employed.Random.seed
     class(objF) <- c("segmented", class(obj0))
     list.obj[[length(list.obj) + 1]] <- objF
